/*
   AVR SERIAL.C (Tiny Serial Driver)
   2004-2007 (C) Mio Software Lab. / Nishiyama, Naoki
   for GCC 3.3 and avr-libc

   ̃t@C͎ςȂĂȂDɂĂ˃CZXłB
   ̂܂܋ARɎgƂł܂B
@ IPI  FTDI-AVR ̃TvQlɂĂ܂B
 
   AVR ̋NbNAgprbg[gɂāA
   serial.h ̒萔ύXKv܂B
@ TX,RX ̃|C^͔r(ύXɊ荞ݓւ̔z)sĂ܂̂ŁA
@ gpɂ͒ӂĂB
*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "serial.h"

/* TX/RX FIFO */
volatile unsigned char RXBUF[RX_BUFFER_SIZE];
volatile unsigned char TXBUF[TX_BUFFER_SIZE];
volatile unsigned char next_in_rx,next_out_rx ;
volatile unsigned char next_in_tx,next_out_tx ;

/* UART0 initialisation */
void uart0_init(void)
{
  /* disable ints */
  UCR = 0x00 ;

  /* set baud rate to header */
  UBRR = UART_BAUD_SELECT ;  

  /* enable RxD/TxD and RX int */
  UCR = _BV(RXCIE) + _BV(TXEN) + _BV(RXEN) ;       
}

/* UART RX INTERRUPT */
SIGNAL(SIG_UART_RECV)      
{
  unsigned char t,c;

  c = UDR ;
  t = next_in_rx ;
  next_in_rx = (next_in_rx + 1) & (RX_BUFFER_SIZE-1) ;
  if (next_in_rx == next_out_rx) { // Buffer full !!
    next_in_rx = t ;
  } else {
    RXBUF[t] = c;
  }
}

/* get one char from recv buffer */
int uart_getc(void) 
{
  unsigned char c;

  /* wait until receive */
  if (next_in_rx == next_out_rx) {
    return -1 ; /* NO DATA */
  }
  c = RXBUF[next_out_rx] ;
  next_out_rx = (next_out_rx + 1)&(RX_BUFFER_SIZE-1);
  return (int)c ;
}

/* put one char to send buffer */
char uart_putc(unsigned char c) 
{
  unsigned char t;

  t = next_in_tx ;
  next_in_tx = (next_in_tx + 1) & (TX_BUFFER_SIZE-1) ;
  if (next_in_tx == next_out_tx) { // Buffer full !!
    next_in_tx = t ;
    return -1 ;			     
  } else {
    TXBUF[t] = c;
    return 0 ;
  }
}

/* send one char to UART (Use Polling) */
char uart_send(void) 
{
  unsigned char c;

  if (next_in_tx != next_out_tx) { /* NOP if no data */
    c = USR ; /* READ STATUS */
    if (c & _BV(UDRE)) { /* NOP if NOT TX READY */
      c = TXBUF[next_out_tx] ;
      next_out_tx = (next_out_tx+1) & (TX_BUFFER_SIZE-1);
      UDR = c ; /* SEND DATA */
    }
    return 0 ;
  }
  return -1 ;
}

/* put strings to send buffer */
char uart_puts(unsigned char *s) 
{
  unsigned char t;

  while((t = *(s++)) != 0) {
    if (uart_putc(t) != 0) {
      return -1 ; // Buffer Full
    }
  }
  return 0 ; // Normal End
}


